import unittest
from unittest.mock import MagicMock
import logging

from hopara.hopara import Hopara
from hopara.table import Table


logging.getLogger('pyhopara').setLevel(logging.WARNING)


class HoparaCase(unittest.TestCase):
    def setUp(self):
        self.hopara = Hopara()
        self.hopara.config = MagicMock()
        self.hopara.config.get_dataset_url = lambda: 'https://test.hopara.py/'
        self.hopara.request = MagicMock()

    def test_urls(self):
        self.assertEqual(self.hopara.get_table_url(Table('table1')), r"https://test.hopara.py/table/table1/")
        self.assertEqual(self.hopara.get_row_url(Table('table1')), r"https://test.hopara.py/table/table1/row?dataSource=hopara")


if __name__ == '__main__':
    unittest.main()



class Filter:
    """
    Used to filter the ``hopara.Table.update_rows`` operation. If support the following types:
    - EQUALS: ``Filter("column1") == [1, 2, 3]``
    - LESS_THAN: ``Filter("column1") < [1, 2, 3]``
    - GREATER_THAN: ``Filter("column1") > [1, 2, 3]``
    - LESS_EQUALS_THAN: ``Filter("column1") <= [1, 2, 3]``
    - GREATER_EQUALS_THAN: ``Filter("column1") >= [1, 2, 3``
    """
    def __init__(self, column_name: str):
        """
        :param column_name: the name of the columns to be filtered
        :type column_name: str
        """
        self.column_name = column_name

    def __dump(self, comparison_type: str, values: list) -> dict:
        if not isinstance(values, list):
            values = [values]
        return {"column": self.column_name, "comparisonType": comparison_type, "values": values}

    def __eq__(self, values: list) -> dict:
        return self.__dump("EQUALS", values)

    def __lt__(self, values: list) -> dict:
        return self.__dump("LESS_THAN", values)

    def __gt__(self, values: list) -> dict:
        return self.__dump("GREATER_THAN", values)

    def __le__(self, values: list) -> dict:
        return self.__dump("LESS_EQUALS_THAN", values)

    def __ge__(self, values: list) -> dict:
        return self.__dump("GREATER_EQUALS_THAN", values)



import os

import toml


class Config:
    def __init__(self):
        self.__config = self.__get_file_config()

    def get_active_env(self) -> str:
        return self.__config.get('default')

    @staticmethod
    def __get_file_config():
        config_path = os.path.join(os.getenv("HOME", ''), '.hopara', 'default.toml')
        if os.path.exists(config_path):
            with open(config_path, 'r') as fp:
                return toml.load(fp)
        return None

    def get_value(self, name):
        return self.__config[self.get_active_env()].get(name)

    def get_auth_url(self) -> str:
        auth_url = os.getenv("HOPARA_AUTH_URL")
        if auth_url is None and self.__config:
            auth_url = self.get_value('authEndpoint')
        return auth_url

    def get_dataset_url(self) -> str:
        dataset_url = os.getenv("HOPARA_DATASET_URL")
        if dataset_url is None and self.__config:
            dataset_url = self.get_value('datasetEndpoint')
        return dataset_url

    def get_app_url(self) -> str:
        app_url = os.getenv("HOPARA_APP_URL")
        if app_url is None and self.__config:
            app_url = self.get_value('appEndpoint')
        return app_url

    def get_resource_url(self) -> str:
        email = os.getenv("HOPARA_RESOURCE_URL")
        if email is None and self.__config:
            email = self.get_value('resourceEndpoint')
        return email

    def get_email(self) -> str:
        email = os.getenv("HOPARA_EMAIL")
        if email is None and self.__config:
            email = self.get_value('email')
        return email

    def get_password(self) -> str:
        password = os.getenv("HOPARA_PASSWORD")
        if password is None and self.__config:
            password = self.get_value('password')
        return password

    @staticmethod
    def get_client_id() -> str:
        return os.getenv("HOPARA_CLIENT_ID")

    @staticmethod
    def get_client_secret() -> str:
        return os.getenv("HOPARA_CLIENT_SECRET")

    def get_credentials(self) -> str:
        client_id, client_secret = self.get_client_id(), self.get_client_secret()
        if client_id and client_secret:
            return {'clientId': client_id, 'clientSecret': client_secret}
        else:
            email, password = self.get_email(), self.get_password()
            if email and password:
                return {'email': email, 'password': password}
        return None


if __name__ == "__main__":
    config = Config()
    print(config.get_dataset_url())
    print(config.get_email())
    print(config.get_client_id())
    print(config.get_client_secret())
    print(config.get_credentials())



import logging

from hopara.filter import Filter
from hopara.hopara import Hopara
from hopara.table import Table
from hopara.view import View
from hopara.type import ColumnType, TypeParam

_log = None


def init_logger():
    global _log
    if _log is None:
        handler = logging.StreamHandler()
        formatter = logging.Formatter('[%(asctime)s - %(name)s - %(levelname)s] %(message)s')
        handler.setFormatter(formatter)
        _log = logging.getLogger('pyhopara')
        _log.addHandler(handler)
        _log.setLevel(logging.INFO)
    return _log


init_logger()



import unittest

from hopara.filter import Filter


class FilterCase(unittest.TestCase):
    def test_filter_json(self):
        self.assertDictEqual(Filter("column1") <= [1, 2, 3],
                             {'column': 'column1', 'comparisonType': 'LESS_EQUALS_THAN', 'values': [1, 2, 3]})
        self.assertDictEqual(Filter("column1") < [1, 2, 3],
                             {'column': 'column1', 'comparisonType': 'LESS_THAN', 'values': [1, 2, 3]})
        self.assertDictEqual(Filter("column1") >= [1, 2, 3],
                             {'column': 'column1', 'comparisonType': 'GREATER_EQUALS_THAN', 'values': [1, 2, 3]})
        self.assertDictEqual(Filter("column1") > [1, 2, 3],
                             {'column': 'column1', 'comparisonType': 'GREATER_THAN', 'values': [1, 2, 3]})
        self.assertDictEqual(Filter("column1") == [1, 2, 3],
                             {'column': 'column1', 'comparisonType': 'EQUALS', 'values': [1, 2, 3]})

    def test_filter_with_no_list_of_values(self):
        self.assertDictEqual(Filter("column1") <= 123,
                             {'column': 'column1', 'comparisonType': 'LESS_EQUALS_THAN', 'values': [123]})



import unittest

import pandas as pd

from hopara.table import Table, ColumnType, TypeParam
from hopara.from_pandas import get_table


class TableCase(unittest.TestCase):
    def setUp(self):
        self.data = {'a': [1, 2, 3],
                     'b': ['a', 'b', 'c'],
                     'c': [1.2, 1.5, 1.7],
                     'd': ['aa', 'bb', 'cc']}
        self.expected_body = {'columns': [{'name': 'a', 'type': 'INTEGER'},
                                            {'name': 'b', 'type': 'STRING'},
                                            {'name': 'c', 'type': 'JSON'},
                                            {'name': 'd', 'type': 'STRING'}],
                              'dataSource': 'hopara'}

    def test_create_table_with_pandas_df(self):
        df = pd.DataFrame(self.data)

        table = get_table('table1', df)
        table.add_column('c', ColumnType.JSON)

        self.assertDictEqual(self.expected_body, table.get_payload())
        self.assertDictEqual(self.expected_body, table.get_payload())

    def test_create_table_manually(self):
        table = Table('table1')
        table.add_column('a', ColumnType.INTEGER)
        table.add_column('b', ColumnType.STRING)
        table.add_column('c', ColumnType.JSON)
        table.add_column('d', ColumnType.STRING)

        self.assertDictEqual(self.expected_body, table.get_payload())

    def test_add_column_checking_column_type(self):
        table = Table('table1')
        table.add_column('a', ColumnType.INTEGER)
        table.add_column('a', None)
        self.assertRaises(AttributeError, table.add_column, 'a', 'INVALID')

    def test_add_column_with_type_param(self):
        table = Table('table1')
        table.add_columns(['a', 'b'], ColumnType.GEOMETRY, TypeParam.GEOMETRY.LINESTRING)
        self.assertListEqual([{'name': 'a', 'type': 'GEOMETRY', 'typeParam': 'LINESTRING'},
                              {'name': 'b', 'type': 'GEOMETRY', 'typeParam': 'LINESTRING'}],
                             table.get_payload()['columns'])



from typing import Union
from hopara.type import ColumnType, TypeParam


class Table:
    """Hopara Table type.
    """
    def __init__(self, name: str, data_source: str = "hopara"):
        """Initialize a table with a name.
        :param name: name of the table.
        :type name: str
        """
        self.name = name
        self.data_source = data_source
        self.__table = {}
        self.__columns = {}

    def get_payload(self) -> dict:
        self.__table['columns'] = [dict(properties, **{'name': name}) for name, properties in self.__columns.items()]
        self.__table['dataSource'] = self.data_source
        return self.__table

    def add_column(self, column_name: str, column_type: ColumnType, type_param: TypeParam = None):
        """Add a new column to the table. If the column already exists, it will be updated.
        :param column_name: name of the column
        :type column_name: str
        :param column_type: type of the column. See more details about these types at hopara.type.ColumnType.
        :type column_type: hopara.type.ColumnType
        :param type_param: subtype of the column. See more details about these types at hopara.type.TypeParam.
        :type type_param: hopara.type.TypeParam
        """
        self.__columns[column_name] = {}
        if column_type:
            self.__columns[column_name]['type'] = column_type.name
            if type_param:
                self.__columns[column_name]['typeParam'] = type_param.name
        if column_name.startswith('_'):
            self.__columns[column_name]['label'] = column_name.lstrip('_')

    def add_columns(self, column_names: list, column_type: ColumnType, type_param: TypeParam = None):
        """Add a list of columns to the table. If the columns already exists, it will be updated.
        :param column_names: list of column names
        :type column_names: list of str
        :param column_type: type of the column. See more details about these types at hopara.type.ColumnType.
        :type column_type: hopara.type.ColumnType
        :param type_param: subtype of the column. See more details about these types at hopara.type.TypeParam.
        :type type_param: hopara.type.TypeParam
        """
        for column_name in column_names:
            self.add_column(column_name, column_type, type_param)

    def __update_column_properties(self, column_name: Union[str, list], properties: dict):
        if isinstance(column_name, str):
            self.__columns[column_name].update(properties)
        else:
            for col_name in column_name:
                self.__columns[col_name].update(properties)

    def set_searchable_columns(self, column_name: Union[str, list]):
        """Set a column as searchable, which means an index will be created to speed up the queries
         :param column_name: column or list of column that must have an index created
         :type column_name: str | list
         """
        self.__update_column_properties(column_name, {'searchable': True})



class View:
    """Hopara View type.
    """
    def __init__(self, name: str, data_source: str = "hopara"):
        """Initialize a table with a name.
        :param name: name of the table.
        :type name: str
        """
        self.name = name
        self.data_source = data_source


import unittest
from datetime import datetime
import pandas as pd
from hopara.from_pandas import get_table, get_rows


class FromPandasCase(unittest.TestCase):
    def setUp(self):
        self.df = pd.DataFrame({'int': [1, 2, 3],
                                'str': ['a', 'b', 'c'],
                                'float': [1.2, 1.5, 1.7],
                                'bool': [True, False, False],
                                'datetime': [datetime(2022, 1, 7), datetime(2022, 4, 6), datetime(2022, 1, 23)]
                                })

    def test_get_table(self):
        table = get_table('from_pandas_test_table', self.df)
        columns = table.get_payload()['columns']
        self.assertListEqual(columns,[{'type': 'INTEGER', 'name': 'int'},
                                      {'type': 'STRING', 'name': 'str'},
                                      {'type': 'DECIMAL', 'name': 'float'},
                                      {'type': 'BOOLEAN', 'name': 'bool'},
                                      {'type': 'DATETIME', 'name': 'datetime'}])

    def test_get_rows(self):
        rows = get_rows(self.df)
        for row in rows:
            row['datetime'] = row['datetime'].rstrip('Z')
        self.assertListEqual(rows, [
            {'int': 1, 'str': 'a', 'float': 1.2, 'bool': True, 'datetime': '2022-01-07T00:00:00.000'},
            {'int': 2, 'str': 'b', 'float': 1.5, 'bool': False, 'datetime': '2022-04-06T00:00:00.000'},
            {'int': 3, 'str': 'c', 'float': 1.7, 'bool': False, 'datetime': '2022-01-23T00:00:00.000'}
        ])


if __name__ == '__main__':
    unittest.main()



import json
import logging
import requests
import os

from requests.compat import urljoin

from hopara.config import Config
from hopara.request import Request
from hopara.table import Table
from hopara.view import View


BATCH_SIZE = 50000


logger = logging.getLogger('pyhopara')


class Hopara:
    """This class handles all data operations such as creating tables and inserting rows.
    """
    def __init__(self, organization: str = None):
        """Initialize with the organization you want to operate.
        Usually the organization is the domain of your work e-mail (e.g. mycompany.com).
        :param organization: Name of the organization.
        :type organization: str
        """
        self.config = Config()
        if organization is None:
            self.__tenant = os.environ.get('HOPARA_TENANT', 'hopara.io')
        else:            
            self.__tenant = organization

        self.request = Request(self.config, {'tenant': self.__tenant})
        logger.info(f'ORGANIZATION: {self.__tenant}')
        logger.info(f'HOST: {self.config.get_dataset_url()}')
        logger.info(f'USER: {self.config.get_client_id() or self.config.get_email()}')

    def get_table_url(self, table: Table) -> str:
        return urljoin(self.config.get_dataset_url(), f'/table/{table.name}/')

    def get_view_url(self, view: View) -> str:
        return urljoin(self.config.get_dataset_url(), f'/view/{view.name}/')

    def get_row_url(self, table: Table) -> str:
        return urljoin(self.get_table_url(table), 'row') + f'?dataSource={table.data_source}' 

    def refresh_stats(self, view: View):
        """ Refresh the stats for a view
        :param view: view name
        """
        url = urljoin(self.get_view_url(view), 'refresh-stats') + f'?dataSource={view.data_source}'
        response = self.request.post(url, {})
        logger.info(f'RESPONSE: {response} / {response.content}\n')

    def delete_table(self, table: Table):
        """ Delete a table
        :param table: table
        :type table: hopara.Table
        """
        url = self.get_table_url(table) + f'?dataSource={table.data_source}' 
        response = self.request.delete(url, table.get_payload())
        logger.info(f'RESPONSE: {response} / {response.content}\n')

    def create_table(self, table: Table, recreate: bool = False):
        """ Create a table
        :param table: table name
        :type table: hopara.Table
        :param recreate: If set to ``True`` the table will be deleted and recreated. If set to ``False`` new columns will be added to the existing table.
        Default: False.
        **If True is set all data previously store in the table will be permanently removed.**
        :type recreate: bool
        """
        if recreate:
            self.delete_table(table)
        url = self.get_table_url(table)
        logger.info(f'URL: {url}')
        logger.info(f'TABLE: {json.dumps(table.get_payload())}')
        response = self.request.post(url, table.get_payload())
        logger.info(f'RESPONSE: {response} / {response.content}\n')

    def __insert_rows(self, url, rows: list):
        response = self.request.post(url, rows)
        logger.info(f'RESPONSE: {response} / {response.content}')

    def insert_rows(self, table: Table, rows: list):
        """ Insert rows in a table.
        :param table: table object
        :type table: hopara.Table
        :param rows: the data to be inserted in the following format: ``[{'col1': 1, 'col2': 'a'}, {'col1': 2, 'col2': 'b'}]``
        """
        url = self.get_row_url(table)
        logger.info(f'URL: {url}')
        logger.info(f'SAMPLE: {json.dumps(rows[0])}')

        if BATCH_SIZE >= len(rows):
            return self.__insert_rows(url, rows)
        batches = range(0, len(rows), BATCH_SIZE)
        logger.info(f'Processing {len(rows):,} rows in {len(batches):,} batches of {BATCH_SIZE:,} rows each...')
        for i, start in enumerate(batches, 1):
            end = min(start + BATCH_SIZE, len(rows))
            self.__insert_rows(url, rows[start:end])

    def __upload_file(self, file_path, resource_type, resource_name=None):
        url = urljoin(self.config.get_resource_url(), f'/tenant/{self.__tenant}/{resource_type}')
        if resource_name:
            url = urljoin(url, f'/{resource_name}')
        with open(file_path, 'rb') as fp:
            # Se é um recurso nomeado, vamos sobrescrever, logo é PUT
            request_func = requests.put if resource_name else requests.post
            response = request_func(url, files={'file': fp}, headers={'tenant': self.__tenant})
            response.raise_for_status()
            logger.info(f'RESPONSE: {response} / {response.content}')
            return response.json()

    def upload_image(self, file_path):
        return self.__upload_file(file_path, 'image')

    def upload_model(self, file_path, resource_name):
        return self.__upload_file(file_path, 'model', resource_name)

    def upload_icon(self, file_path, resource_name):
        return self.__upload_file(file_path, 'icon', resource_name)


if __name__ == "__main__":
    hopara = Hopara()



from enum import Enum, auto


class ColumnType(Enum):
    """hopara.Table support all the following types for its columns:
     - ``STRING``, ``INTEGER``, ``DECIMAL``, ``BOOLEAN``
     - ``DATETIME``: python datetime format
     - ``AUTO_INCREMENT``: auto-increment integer
     - ``MONEY``: currency values
     - ``JSON``: json column, can be used for regular JSON or a (GeoJson)(https://geojson.org/)
     - ``STRING_ARRAY``: string array column: ``['value1', 'value2']``
     - ``GEOMETRY``: a coordinates array representing a Geometry ``[[0, 0], [1, 0], [1, 1], [0, 1], [0, 0]]``
    """
    STRING = auto()
    INTEGER = auto()
    DECIMAL = auto()
    BOOLEAN = auto()
    DATETIME = auto()
    AUTO_INCREMENT = auto()
    MONEY = auto()
    JSON = auto()
    STRING_ARRAY = auto()
    GEOMETRY = auto()
    IMAGE = auto()


class TypeParam:
    """Some types support additional parameters to describe the type.
         - ``GEOMETRY`` supports: ``LINESTRING``
    """
    class GEOMETRY(Enum):
        LINESTRING = auto()



import os

from setuptools import find_packages, setup

current_dir = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(current_dir, 'requirements.txt')) as fp:
    requires = fp.read().splitlines()

with open(os.path.join(current_dir, 'README.md')) as fp:
    long_description = fp.read()

setup(
    name='hopara',
    packages=find_packages(include=['hopara']),
    version='1.15.0',
    description='Hopara Python Library',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Hopara Inc',
    install_requires=requires
)



import json

import requests
from requests.compat import urljoin
import logging


logger = logging.getLogger('pyhopara')


class Request:
    def __init__(self, config: object, header: dict = None):
        self.config = config
        self.__access_token = None
        self.__header = header if header else {}

    def __get_headers(self) -> dict:
        self.__header['Authorization'] = 'Bearer ' + self.get_access_token()
        return self.__header

    def get_access_token(self) -> str:
        if self.__access_token:
            return self.__access_token
        url = urljoin(self.config.get_auth_url(), 'token')
        response = requests.post(url, json=self.config.get_credentials())
        response.raise_for_status()
        self.__access_token = response.json()['access_token']
        return self.__access_token

    @staticmethod
    def __handle_response(response: requests.Response, ignore_error: bool = False) -> requests.Response:
        if 200 <= response.status_code <= 299 or ignore_error:
            return response
        if response.headers.get('Content-Type') == 'application/json':
            logger.debug(json.dumps(response.json(), indent=True))
        else:
            logger.debug(response.content)
        response.reason = response.content
        response.raise_for_status()

    def post(self, url: str, body: dict) -> requests.Response:
        return self.__handle_response(requests.post(url, json=body, headers=self.__get_headers()))

    def get(self, url: str, body: dict) -> requests.Response:
        return self.__handle_response(requests.get(url, json=body, headers=self.__get_headers()))

    def delete(self, url: str, body: dict) -> requests.Response:
        return self.__handle_response(requests.delete(url, json=body, headers=self.__get_headers()), ignore_error=True)

    def put(self, url: str, body: dict) -> requests.Response:
        return self.__handle_response(requests.put(url, json=body, headers=self.__get_headers()))



import json
import pandas as pd
import pandas.api.types as pdt
from hopara import Table, ColumnType


def get_rows(df: pd.DataFrame) -> list:
    """Get rows in Hopara format from a Pandas df.
    :param df: a DataFrame from pandas library.
    :type df: pandas.DataFrame
    :return: rows in Hopara format
    :rtype: list of dicts
    """
    return json.loads(df.to_json(orient="records", date_format='iso'))


def get_table(table_name: str, df: pd.DataFrame, data_source:str = "hopara") -> Table:
    """Generate a Hopara Table from a Pandas df.
    This function is able to detect the most common types.
    When the type detection fails you can manually set the type by calling the ``add_column`` function on the table object.

    Auto-detected types
     - ``STRING``, ``INTEGER``, ``DECIMAL``, ``BOOLEAN``
     - ``DATETIME``: python datetime format

    :param table_name: table name
    :type table_name: str
    :param df: pandas DataFrame
    :type df: pandas.DataFrame
    :return: Table generated based on pandas DataFrame
    :rtype: hopara.Table
    """
    table = Table(table_name, data_source)
    for column_name in df.columns:
        column_type = None
        if pdt.is_float_dtype(df[column_name]):
            column_type = ColumnType.DECIMAL
        elif pdt.is_integer_dtype(df[column_name]):
            column_type = ColumnType.INTEGER
        elif pdt.is_bool_dtype(df[column_name]):
            column_type = ColumnType.BOOLEAN
        elif pdt.is_string_dtype(df[column_name]):
            column_type = ColumnType.STRING
        elif pdt.is_datetime64_dtype(df[column_name]):
            column_type = ColumnType.DATETIME
        table.add_column(column_name, column_type)
    return table

